/*
	File:		CDIL.h, v2.0a2

	Contains:	Public interface to the Desktop Integration Library

	Copyright:	Apple Computer, Inc. 1997, all rights reserved.
*/

#ifndef __CDIL_H
#define __CDIL_H

#include "DIL.h"

#ifdef __cplusplus
extern "C" {
#endif

/* --- CDIL error numbers --- */

#define kCD_CDILNotInitialized			(kCD_ErrorBase - 1)
#define kCD_ServiceNotSupported			(kCD_ErrorBase - 2)
#define kCD_BadPipeState				(kCD_ErrorBase - 3)
#define kCD_Timeout						(kCD_ErrorBase - 4)
#define kCD_PipeDisconnected			(kCD_ErrorBase - 5)
#define kCD_IndexOutOfRange				(kCD_ErrorBase - 6)
#define kCD_BufferTooSmall				(kCD_ErrorBase - 7)
#define kCD_PlatformError				(kCD_ErrorBase - 8)

#define kCD_TCPCantFindLibraryFns		(kCD_ErrorBase - 20)
#define kCD_TCPInsufficientVersion		(kCD_ErrorBase - 21)
#define kCD_TCPNoSockets				(kCD_ErrorBase - 22)

typedef enum CD_State
{
	kCD_Unknown,
	kCD_Uninitialized,
	kCD_Disconnected,
	kCD_Listening,
	kCD_ConnectPending,
	kCD_Connected,
	kCD_DisconnectPending
} CD_State;

/*
**	Values that can be passed to CD_SetTimeout as timeout values.
*/

#define kCD_DefaultTimeout				(-1)		/* 30 seconds			*/
#define kCD_NoTimeout					(-2)		/* many more seconds	*/


DECLARE_DIL_HANDLE(CD_Handle);

DILAPI DIL_Error	CD_Startup			(void);
DILAPI DIL_Error	CD_Shutdown			(void);

DILAPI DIL_Error	CD_HasADSP			(void);
DILAPI DIL_Error	CD_HasCTB			(const char* toolName);
DILAPI DIL_Error	CD_HasMNPSerial		(void);
DILAPI DIL_Error	CD_HasTCP			(void);

DILAPI DIL_Error	CD_GetSerialPortName(long index, char* buffer, long* bufLen);

DILAPI DIL_Error	CD_CreateADSP		(CD_Handle*, const char* name, const char* type);
DILAPI DIL_Error	CD_CreateCTB		(CD_Handle*, const char* toolName, const char* configString);
DILAPI DIL_Error	CD_CreateMNPSerial	(CD_Handle*, long port, long baud);
DILAPI DIL_Error	CD_CreateTCP		(CD_Handle*, long port);

DILAPI DIL_Error	CD_Dispose			(CD_Handle);

DILAPI DIL_Error	CD_Disconnect		(CD_Handle);
DILAPI DIL_Error	CD_StartListening	(CD_Handle);
DILAPI DIL_Error	CD_Accept			(CD_Handle);
DILAPI DIL_Error	CD_Read				(CD_Handle, void* p, long count);
DILAPI DIL_Error	CD_BytesAvailable	(CD_Handle, long* count);
DILAPI DIL_Error	CD_Write			(CD_Handle, const void* p, long count);
DILAPI DIL_Error	CD_FlushOutput		(CD_Handle);
DILAPI DIL_Error	CD_Idle				(CD_Handle);

DILAPI CD_State	CD_GetState			(CD_Handle);

DILAPI DIL_Error	CD_SetTimeout		(CD_Handle, long timeoutInSecs);

DILAPI long		CD_GetPlatformError	(CD_Handle);

#ifdef __cplusplus
}
#endif

#endif	/* __CDIL_H */


